//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from Arm Limited or its affiliates.
//
//            (C) COPYRIGHT 2013 Arm Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from Arm Limited or its affiliates.
//
//      SVN Information
//
//      Checked In          : $Date: 2017-10-10 15:55:38 +0100 (Tue, 10 Oct 2017) $
//
//      Revision            : $Revision: 371321 $
//
//      Release Information : Cortex-M System Design Kit-r1p1-00rel0
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Abstract : Synchroniser for MTB Control (TSTART, TSTOP)
//-----------------------------------------------------------------------------
//

module cmsdk_mtb_sync(
    // Free running clock
    input wire              FCLK,

    // Global reset
    input wire              HRESETn,

    // MTB Control
    input wire              TSTART,          // External Trace Start pin
    input wire              TSTOP,           // External Trace Stop pin

    // MTB Control
    output wire             TSTART_SYNC,     // Synchronised Trace Start pin
    output wire             TSTOP_SYNC);     // Select MTB Special Function Registers

  // Signals for input double flop-flop synchroniser
  reg    [1:0] reg_in_sync1;
  reg    [1:0] reg_in_sync2;

  // ----------------------------------------------------------
  // Synchonize input with double stage flip-flops
  // ----------------------------------------------------------

  always @(posedge FCLK or negedge HRESETn)
  begin
    if (~HRESETn)
      begin
      reg_in_sync1 <= {2{1'b0}};
      reg_in_sync2 <= {2{1'b0}};
      end
    else
      begin
      reg_in_sync1 <= {TSTART, TSTOP};
      reg_in_sync2 <= reg_in_sync1;
      end
  end

  assign TSTART_SYNC = reg_in_sync2[1];
  assign TSTOP_SYNC  = reg_in_sync2[0];

endmodule
